﻿using Hotline.CallCenter.Calls;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Enums.CallCenter;
using Hotline.Share.Enums.User;
using Hotline.Share.Requests;
using Hotline.Users;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers.Bi;

/// <summary>
/// 话务报表
/// </summary>
public class BiCallController : BaseController
{
    private readonly IRepository<TrCallRecord> _trCallRecordRepository;
    private readonly IRepository<User> _userRepository;

    public BiCallController(
        IRepository<TrCallRecord> trCallRecordRepository,
        IRepository<User> userRepository)
    {
        _trCallRecordRepository = trCallRecordRepository;
        _userRepository = userRepository;
    }

    [HttpGet("calls")]
    [AllowAnonymous]
    public async Task<IReadOnlyList<BiCallDto>> QueryCallsAsync([FromQuery] BiQueryCallsDto dto)
    {
        dto.StartTime ??= DateTime.Today;
        dto.EndTime ??= DateTime.Today.AddDays(1).AddSeconds(-1);

        var items = await _trCallRecordRepository.Queryable()
            .Where(d => d.CreatedTime >= dto.StartTime && d.CreatedTime <= dto.EndTime)
            .Where(d => d.CallDirection == ECallDirection.In)
            .WhereIF(!string.IsNullOrEmpty(dto.Line), d => d.Gateway == dto.Line)
            .Select(d => new
            {
                d.CreatedTime.Year,
                d.CreatedTime.Month,
                d.CreatedTime.Day,
                d.CreatedTime.Hour,
                d.AnsweredTime,
                d.EndBy
            })
            .MergeTable()
            .GroupBy(d => new
            {
                d.Year,
                d.Month,
                d.Day,
                d.Hour
            })
            .Select(d => new BiCallDto
            {
                Hour = d.Hour,
                Total = SqlFunc.AggregateCount(d.Hour),
                Answered = SqlFunc.AggregateSum(SqlFunc.IIF(d.AnsweredTime != null, 1, 0)),
                Hanguped = SqlFunc.AggregateSum(SqlFunc.IIF(d.AnsweredTime == null && d.EndBy != null && d.EndBy.Value == EEndBy.To, 1, 0))
            })
            .OrderBy(d => d.Hour)
            .ToListAsync(HttpContext.RequestAborted);

        if (items.Count < 24)
        {
            for (var i = 0; i < 24; i++)
            {
                var item = items.FirstOrDefault(d => d.Hour == i);
                if (item is null)
                    items.Add(new BiCallDto { Hour = i });
            }

            items = items.OrderBy(d => d.Hour).ToList();
        }

        return items;
    }

    [HttpGet("seats")]
    [AllowAnonymous]
    public async Task<IReadOnlyList<BiSeatCallsDto>> QuerySeatCallsAsync([FromQuery] ReportPagedRequest dto)
    {
        //from config
        var hangupSeconds = 5;

        return await _userRepository.Queryable()
              .LeftJoin<TrCallRecord>((u, c) => u.Id == c.UserId)
              .Where(u => !u.IsDeleted && u.UserType == EUserType.Seat)
              .WhereIF(dto.StartTime.HasValue, (_, c) => c.CreatedTime >= dto.StartTime.Value)
              .WhereIF(dto.EndTime.HasValue, (_, c) => c.CreatedTime <= dto.EndTime.Value)
              .GroupBy((u, c) => new { c.UserName, c.TelNo })
              .Select((u, c) => new BiSeatCallsDto
              {
                  Name = c.UserName,
                  TelNo = c.TelNo,
                  InTotal = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In, 1, 0)),
                  OutTotal = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.Out, 1, 0)),
                  InAnswered = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime != null, 1, 0)),
                  OutAnswered = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.Out && c.AnsweredTime != null, 1, 0)),
                  InHangupImmediate = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime == null && c.RingTimes < hangupSeconds, 1, 0)),
                  InHanguped = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime == null, 1, 0)),
                  InDurationAvg = SqlFunc.AggregateAvg(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime != null, c.Duration, 0)),
                  OutDurationAvg = SqlFunc.AggregateAvg(SqlFunc.IIF(c.CallDirection == ECallDirection.Out && c.AnsweredTime != null, c.Duration, 0)),
                  InAvailableAnswer = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime != null && c.Duration >= hangupSeconds, 1, 0)),
                  InHangupImmediateWhenAnswered = SqlFunc.AggregateSum(SqlFunc.IIF(c.CallDirection == ECallDirection.In && c.AnsweredTime != null && c.Duration < hangupSeconds, 1, 0)),
              })
              .MergeTable()
              .ToListAsync(HttpContext.RequestAborted);
    }
}
